@@ -48,10 +48,9 @@ void cmdline_write_history()
 
     char* histfile = getenv("HISTORY_FILE");
     if(histfile) 
-        strcpy(path, histfile);
+        strncpy(path, histfile, PATH_MAX);
     else {
-        strcpy(path, gHomeDir);
-        strcat(path, "/history");
+        snprintf(path, PATH_MAX, "%s/history", gHomeDir);
     }
 
     if(access(path, W_OK) != 0) {
@@ -100,17 +99,44 @@ void cmdline_read_history()
         exit(1);
     }
     char path[PATH_MAX];
-    strcpy(path, env);
+    strncpy(path, env, PATH_MAX);
 
     if(access(path, R_OK) == 0) {
-        char line[kCmdLineMax];
+        string_obj* line = STRING_NEW("");
 
-        FILE* f = fopen(path, "r+");
-        while(fgets(line, kCmdLineMax, f) != NULL) {
-            line[strlen(line)-1] = 0; // chomp
-            vector_add(gHistory, STRING_NEW(line));
+        int fd = open(path, O_RDONLY);
+        while(1) {
+            char buf[BUFSIZ];
+            int size = read(fd, buf, BUFSIZ);
+
+            if(size <= 0) {
+                break;
+            }
+            else if(size > 0) {
+                buf[size] = 0;
+                string_push_back(line, buf);
+            }
         }
-        fclose(f);
+        close(fd);
+
+        while(1) {
+            char* p = strstr(string_c_str(line), "\n");
+            if(p == NULL) {
+                break;
+            }
+            else {
+                int size2 = p - string_c_str(line);
+                char* line2 = MALLOC(size2+1);
+                memcpy(line2, string_c_str(line), size2);
+                line2[size2] = 0;
+
+                vector_add(gHistory, STRING_NEW(line2));
+                FREE(line2);
+                string_erase(line, 0, size2+1);
+            }
+        }
+
+        string_delete(line);
     }
 }
 
@@ -123,7 +149,7 @@ void cmdline_init()
 {
     gCmdLine = STRING_NEW("");
     gCmdLineSaved = STRING_NEW("");
-    gHistory = VECTOR_NEW(1);
+    gHistory = VECTOR_NEW(100);
     gHCandidate = VECTOR_NEW(1);
 }
 
@@ -154,6 +180,10 @@ void cmdline_start(char* str, int cursor, BOOL quick, BOOL continue_)
     gCmdLineHitAnyKey = !quick;
     gCmdLineContinue = continue_;
     string_put(gCmdLine, str);
+    if(string_length(gCmdLine) > kCmdLineMax) {
+        string_trunc(gCmdLine, kCmdLineMax);
+    }
+    cmdline_convert();
 
     /// cursorは文字列単位での位置を指定してくるのでバイト単位に変換する
     if(cursor >= 0 && cursor < str_kanjilen(kUtf8, string_c_str(gCmdLine))) {
@@ -196,8 +226,6 @@ void cmdline_start(char* str, int cursor, BOOL quick, BOOL continue_)
 
     gHistoryScrollTop = 0;
     gHistoryCursor = -1;
-
-    cmdline_convert();
 }
 
 ///////////////////////////////////////////////////
@@ -221,29 +249,31 @@ void cmdline_convert()
 {
     if(gKanjiCodeFileName != gKanjiCode) {
         char* src = string_c_str(gCmdLine);
-        char* des = MALLOC(strlen(src)*2);
-        int des_size = strlen(src)*2;
+        int des_size = strlen(src)*2 + 1;
+        char* des = MALLOC(des_size + 1);
 #if defined(HAVE_ICONV_H) || defined(HAVE_BICONV_H) 
         if(is_all_ascii(src)) {
-            strcpy(des, src);
+            strncpy(des, src, des_size);
         }
         else {
             /// コンバート ///
             if(kanji_convert(src, des, des_size, gKanjiCodeFileName, gKanjiCode) < 0) {
-                strcpy(des, src);
+                strncpy(des, src, des_size);
             }
         }
 #else
-        strcpy(des, src);
+        strncpy(des, src, des_size);
 #endif
         string_put(gCmdLine, des);
+        if(string_length(gCmdLine) > kCmdLineMax) {
+            string_trunc(gCmdLine, kCmdLineMax);
+        }
         FREE(des);
     }
 }
 
 void cmdline_input(int meta, int key)
 {
-    BOOL kanji[kCmdLineMax];
     char* p = string_c_str(gCmdLine);
     int len = strlen(p);
     int i;
@@ -251,17 +281,6 @@ void cmdline_input(int meta, int key)
 
     BOOL cmdline_escape_key = env && strcmp(env, "1") == 0;
 
-    for(i=0; i<len; i++) {
-        if(is_kanji(kUtf8, p[i])) {
-            kanji[i] = TRUE;
-            kanji[i+1] = FALSE;
-            i++;
-        }
-        else {
-            kanji[i] = FALSE;
-        }
-    }
-
     if(key == 14 || key == KEY_DOWN) {    // CTRL-N
         if(gHistoryCursor == 0) {
             string_delete(gCmdLine);
@@ -605,14 +624,14 @@ void cmdline_input(int meta, int key)
         if(meta == 0 && !(key >= 0 && key <= 27)) {
             char tmp[128];
 
-            sprintf(tmp, "%c", key);
+            snprintf(tmp, 128, "%c", key);
             string_insert(str, gCmdLineCursor, tmp);
             gCmdLineCursor++;
         }
 
         gHistoryCursor = -1;
     }
-
+//    cmdline_convert();
 
     if(gHistoryCursor == -1) {
         for(i=0; i<vector_size(gHCandidate); i++) {
@@ -654,7 +673,9 @@ void cmdline_input(int meta, int key)
         }
     }
 
-    cmdline_convert();
+    if(string_length(gCmdLine) > kCmdLineMax) {
+        string_trunc(gCmdLine, kCmdLineMax);
+    }
 }
 
 void cmdline_hcandidate_refresh()
@@ -701,9 +722,10 @@ static void cmdline_view_utf8_multi_line()
     while(1) {
         char* line_end = str_kanjipos2pointer(kUtf8, line_top, n);
 
-        char buf[4089];
-        memcpy(buf, line_top, line_end-line_top);
-        buf[line_end-line_top] = 0;
+        int size = line_end-line_top;
+        char* buf = MALLOC(size+1);
+        memcpy(buf, line_top, size);
+        buf[size] = 0;
 
         /// カーソル描写位置を保存 ///
         if(line_end == cstr+gCmdLineCursor) {
@@ -713,6 +735,7 @@ static void cmdline_view_utf8_multi_line()
         
         /// 行の限界まできたら ///
         else if(line >= maxy) {
+            FREE(buf);
             break;
         }
 
@@ -722,19 +745,23 @@ static void cmdline_view_utf8_multi_line()
         {
             line_end = str_kanjipos2pointer(kUtf8, line_top, n-1);
 
-            memcpy(buf, line_top, line_end-line_top);
-            buf[line_end-line_top] = 0;
+            int size = line_end-line_top;
+            char* buf2 = MALLOC(size+1);
+            memcpy(buf2, line_top, line_end-line_top);
+            buf2[line_end-line_top] = 0;
 
             if(line == 0) {
-                mprintw("%s", buf);
+                mprintw("%s", buf2);
             }
             else {
-                mmvprintw(line, 0, "%s", buf);
+                mmvprintw(line, 0, "%s", buf2);
             }
 
             line_top = line_end;
             n = 0;
             line++;
+
+            FREE(buf2);
         }
 
         /// 文字列の終端まで達していたら ///
@@ -746,6 +773,8 @@ static void cmdline_view_utf8_multi_line()
 
             mmvprintw(line, 0, "%s", line_top);
 
+            FREE(buf);
+
             break;
         }
 
@@ -753,6 +782,8 @@ static void cmdline_view_utf8_multi_line()
         else {
             n++;
         }
+
+        FREE(buf);
     }
 
     /// カーソル移動 ///
@@ -799,13 +830,15 @@ static void cmdline_view_utf8_2line()
                 char* pointer = str_kanjipos2pointer(kUtf8, cstr, i);
 
                 if(is_kanji(kUtf8, *pointer)) {
-                    char line1[2048];
+                    int size = pointer-cstr;
+                    char* line1 = MALLOC(size+1);
 
-                    memcpy(line1, cstr, pointer-cstr);
-                    line1[pointer-cstr] = 0;
+                    memcpy(line1, cstr, size);
+                    line1[size+1] = 0;
 
                     mmvprintw(maxy-2, 2, "%s", line1);
                     mmvprintw(maxy-1, 0, "%s", pointer);
+                    FREE(line1);
                     break;
                 }
             }
@@ -885,8 +918,6 @@ static void cmdline_view_filer()
 
         mmvprintw(maxy -2, 0, "%s", string_c_str(str));
 
-        if(gISearchExplore) isearch_explore_view();
-
         string_delete(str);
     }
 
@@ -895,7 +926,7 @@ static void cmdline_view_filer()
     sFile* file = filer_cursor_file(adir());
 
     char permission[12];
-    strcpy(permission, "----------");
+    strncpy(permission, "----------", 12);
 
     if(S_ISDIR(file->mLStat.st_mode)) permission[0] = 'd';  // ファイルの種別
     if(S_ISCHR(file->mLStat.st_mode)) permission[0] = 'c';
@@ -936,7 +967,7 @@ static void cmdline_view_filer()
         str_cut2(kUtf8, tmp, 8, owner, 256);
     }
     else {
-        sprintf(owner, "%d", file->mLStat.st_uid);
+        snprintf(owner, 256, "%d", file->mLStat.st_uid);
     }
 
     char group[256];
@@ -945,30 +976,30 @@ static void cmdline_view_filer()
         str_cut2(kUtf8, tmp, 7, group, 256);
     }
     else {
-        sprintf(group, "%d", file->mLStat.st_gid);
+        snprintf(group, 256, "%d", file->mLStat.st_gid);
     }
 
     char size_buf[256];
     int width = filer_make_size_str(size_buf, file->mLStat.st_size);
 
     if(S_ISDIR(file->mStat.st_mode)) {
-        strcpy(size_buf, "<DIR>");
+        strncpy(size_buf, "<DIR>", 256);
     }
 
     char format[128];
-    strcpy(format, "%");
-    sprintf(format + 1, "%d", width);
-    strcat(format, "s");
+    strncpy(format, "%", 128);
+    snprintf(format + 1, 126, "%d", width);
+    strncat(format, "s", 128);
 
     char size_buf2[256];
-    sprintf(size_buf2, format, size_buf);
+    snprintf(size_buf2, 256, format, size_buf);
     if(!S_ISLNK(file->mLStat.st_mode)) {
         int year = tm_->tm_year-100;
         if(year < 0) year+=100;
         while(year > 100) year-=100;
         
-        sprintf(buf2
-            , "%s %3d %-8s %-7s%s %02d-%02d-%02d %02d:%02d %s"
+        snprintf(buf2, 1024, 
+            "%s %3d %-8s %-7s%s %02d-%02d-%02d %02d:%02d %s"
             //, "%s %3d %-8s %-7s%10lld %02d-%02d-%02d %02d:%02d %s"
             , permission, file->mLStat.st_nlink
             , owner, group
@@ -985,8 +1016,8 @@ static void cmdline_view_filer()
         mmvprintw(maxy-1, 0, "%s", buf3);
     }
     else {
-        sprintf(buf
-           , "%s %3d %s%s%s %02d-%02d-%02d %02d:%02d %s -> %s"
+        snprintf(buf, 1024,
+           "%s %3d %s%s%s %02d-%02d-%02d %02d:%02d %s -> %s"
            //, "%s %3d %s%s%10lld %02d-%02d-%02d %02d:%02d %s -> %s"
            , permission, file->mLStat.st_nlink
            , owner, group
@@ -1027,15 +1058,15 @@ void cmdline_history_view()
     for(i= gHistoryScrollTop; i<vector_size(gHCandidate); i++) {
         string_obj* item = (string_obj*)vector_item(gHCandidate, i);
 
-        char* tmp = (char*)malloc(sizeof(char)*kCmdLineMax);
-        str_cut2(kUtf8, string_c_str(item), maxx-1, tmp, kCmdLineMax);
+        char* tmp = (char*)MALLOC(MB_CUR_MAX*maxx+1);
+        str_cut2(kUtf8, string_c_str(item), maxx-1, tmp, MB_CUR_MAX*maxx+1);
 
         if(gHistoryCursor == i) mattron(kCAReverse);        
         mmvprintw(line, 0, "%s", tmp);
         if(gHistoryCursor == i) mattroff();
         line--;
         
-        free(tmp);
+        FREE(tmp);
 
         if(line < 0) {
             break;
