@@ -830,8 +830,13 @@ int main(int argc, char* argv[])
     /// メモリリークの監視を開始 ///
     CHECKML_BEGIN();
 
+#if defined(HAVE_GC_H) || defined(HAVE_GC_GC_H)
+    /// GC 開始 ///
+    GC_init();
+#endif
+
     /// ファイラでもスクリプトでも共通の環境変数を初期化 ///
-    setenv("VERSION", "4.2.7", 1);
+    setenv("VERSION", "4.2.8", 1);
 
     /// mfiler3のホームディレクトリを保存しておく ///
     char* home = getenv("HOME");
