@@ -83,7 +83,7 @@ void menu_final()
 void menu_append(sMenu* self, char* name, int key, char* cmd)
 {
     char buf[256];
-    sprintf(buf, " %-40s", name);
+    snprintf(buf, 256, " %-40s", name);
     vector_add(self->mMenuItems, sMenuItem_new(buf, key, cmd));
 }
 
@@ -243,7 +243,7 @@ void menu_view(sMenu* self)
             str_cut2(kUtf8, item->mName, maxx-4, buf, 1024);
         }
         else {
-            strcpy(buf, item->mName);
+            strncpy(buf, item->mName, 1024);
         }
         
         if(i == self->mCursor) {
